unit comafilter;
               //(version 3.0 or later)
interface

uses windows, sysutils, math, dialogs, forms;

function pi_initialize(funcaddress:pointer; menuname,description:pchar;
                      whandle:HWND):integer; stdcall;
procedure pi_activate(pimage:pointer; whandle:HWND); stdcall;
procedure pi_finalize(whandle:HWND); stdcall;

implementation

const
 PACTIVEIMAGE = 4096;
 PSELECTIMAGE = 4097;
 PEQU_STD = 1000;
 PEQU_GAU = 1001;
 PCGREEN = 0;
 PCRED = 2;
 BCBLUE = 3;

type

Tcommand = (ac_getbuffer, ac_getsize, ac_redraw, ac_close, ac_savefits,
            ac_copy, ac_writeheader, ac_readheader, ac_prepareundo,
            ac_thresholds, ac_transferfunc, ac_getselpoint, ac_getselpoint2,
            ac_gethwnd, ac_create, ac_open, ac_getimage, ac_getversion,
            ac_rename, ac_getname, ac_getra, ac_getdec, ac_getx, ac_gety,
            ac_modified);

Tfloatarray = array[0..$fffffff] of single;
pfloatarray = ^Tfloatarray;
Tcallbackfunc = function(command:Tcommand; p1,p2,p3:pointer):integer; stdcall;

(* Commands: see docs

NAME             PARAMS TYPE                   RESULT

ac_getbuffer    (pimage, &buffer, colorplane)  0,2
ac_getsize      (pimage, &x, &y)               0,1
ac_redraw       (pimage, calcstats, autovis)   0,1
ac_close        (pimage, askuser, nil)         0,1
ac_savefits     (pimage, pfilename, nil)       0,1
ac_copy         (pimage, nil, nil)             0,1
ac_writeheader  (pimage, pheader, nil)         0,1
ac_readheader   (pimage, nil, nil)             pheader
ac_prepareundo  (pimage, nil, nil)             0,1
ac_thresholds   (pimage, &min, &max)           0,1
ac_transferfunc (pimage, trf, nil)             0,1
ac_getselpoint  (pimage, &x, &y)               0,1
ac_getselpoint2 (pimage, &x, &y)               0,1
ac_gethwnd      (pimage, &HWND, nil)           0,1
ac_create       (x,y,colored)                  pimage
ac_open         (pfilename, nil, nil)          pimage
ac_getimage     (selection, nil, nil)          pimage

ac_getversion   (nil, nil, nil)                300+
ac_rename       (pimage, name, path)           0,1
ac_getname      (pimage, path, nil)            pfilename
ac_getra        (pimage, x*100, y*100)         ra*1e5
ac_getdec       (pimage, x*100, y*100)         dec*1e5
ac_getx         (pimage, ra*1e6, dec*1e6)      x*100
ac_gety         (pimage, ra*1e6, dec*1e6)      y*100
ac_modified     (pimage, modified, nil)        0,1
*)


var
 callback: Tcallbackfunc;
 dimx,dimy: integer;
 bg: single; // Calculated background value
 newbuffer,oldbuffer: pfloatarray;


// Utility functions. Not optimized, for learning purposes.

function getPixel(x,y: integer): single;
begin
   if x < 0 then x := 0;
   if y < 0 then y := 0;
   if x >= dimx then x := dimx-1;
   if y >= dimy then y := dimy-1;
   result := oldbuffer[x+y*dimx];
end;

procedure setPixel(x,y: integer; v:single);
begin
   if (x<0) or (y<0) or (x >= dimx) or (y >= dimy) then exit;
   newbuffer[x+y*dimx] := v;
end;

function calcBG: single;
var
  x,y: integer;
  valpix,avg: single;
begin
  valpix := 0.0;
for y := 0 to dimy-1 do
    for x := 0 to dimx-1 do
      valpix := valpix + getPixel(x,y);
      avg := valpix/(dimx*dimy);
      calcBG := avg;
    //  ShowMessage('Average Pixel Value ='+ FloatToStr(avg));
end;

// Function to compute the distance between two pixels
//
function distPixel(x,y,xz,yz: integer): single;
begin
  result := sqrt(power((xz-x),2)+power((yz-y),2));
end;


// Functions exported by the DLL

function pi_initialize(funcaddress:pointer; menuname,description:pchar;
         whandle: HWND):integer; stdcall;
begin
   callback := funcaddress;
   strcopy(menuname,'Radial Weighted Model');
   strcopy(description,'Syntetize a radial weighted model from a real image');
   result := 1;
end;

procedure pi_finalize(whandle:HWND); stdcall;
begin

end;

procedure pi_activate(pimage:pointer; whandle:HWND); stdcall;

var
 pnewimage: pointer;
 x,y,xz,yz,b: integer;
 valpix,peak: single;

begin
  xz := 0;
  yz := 0;
  valpix := 0;
  peak := 0.0;

   if pimage = nil then begin
      messagebox(0,'You must open an image!','ERROR !',0);   // no images opened
      exit;
   end;

   callback(ac_getsize,pimage,@dimx,@dimy);   // get image size
   callback(ac_prepareundo,pimage,nil,nil);
   getmem(oldbuffer,dimx*dimy*4);
   callback(ac_getselpoint,pimage,@xz,@yz);

   b := callback(ac_getbuffer,pimage,@newbuffer,pointer(PCGREEN));  //***
   if b <> 2 then exit;     // buffer is not 32 bit, old version of Astroart
   copymemory(oldbuffer,newbuffer,dimx*dimy*4); // copy image to  an another buffer

   pnewimage := pointer(callback(ac_create,pointer(dimx),pointer(dimy),nil));
   b := callback(ac_getbuffer,pnewimage,@newbuffer,pointer(PCGREEN));
   if b <> 2 then exit;

   if (xz = 0) and (yz = 0) then begin
     messagebox(0,'You must select a pixel as coma centroid','ERROR',0);
     exit;
   end;

   peak := getPixel(xz,yz); // Peak value in the coma image
   bg := calcBG;            // Assign to bg the calculated background
   
   for y := 0 to dimy-1 do
    for x := 0 to dimx-1 do
    begin

      valpix := ((getPixel(x,y)-bg)*distPixel(x,y,xz,yz));

      if valpix <= bg then valpix := bg;
      setPixel(x,y,valpix);

    end;

   //*** on color images you should repeat these steps for other color planes.

   freemem(oldbuffer);
   callback(ac_redraw,pimage,pointer(1),pointer(0));  // update the filtered image
end;


end.

